<?php
/* ======================================================
 # Virtuemart Sales for Joomla! - v7.1.8 (free version)
 # -------------------------------------------------------
 # For Joomla! CMS (v4.x)
 # Author: Web357 (Yiannis Christodoulou)
 # Copyright (©) 2014-2023 Web357. All rights reserved.
 # License: GNU/GPLv3, http://www.gnu.org/licenses/gpl-3.0.html
 # Website: https:/www.web357.com
 # Demo: https://demo.web357.com/joomla/virtuemart-sales
 # Support: support@web357.com
 # Last modified: Monday 10 July 2023, 03:54:31 AM
 ========================================================= */
defined( '_JEXEC' ) or die( 'Restricted access' );

jimport('joomla.application.component.modellist');

class vmsalesModelvmsales extends JModelList {

	/**
	 * Search total
	 *
	 * @var integer
	 */
	protected $_total = null;

	protected $_lang_prefix = null;

	/**
	 * Constructor.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @see        JController
	 * @since      1.6
	 */
	public function __construct($config = array())
	{
		parent::__construct($config);
	}

	/**
	 * Method to auto-populate the model state.
	 *
	 * Note. Calling getState in this method will result in recursion.
	 *
	 * @param   string  $ordering   Elements order
	 * @param   string  $direction  Order direction
	 *
	 * @return void
	 *
	 * @throws Exception
	 *
	 * @since    1.6
	 */
	protected function populateState($ordering = null, $direction = null)
	{
		// Get Params
		$app 	= JFactory::getApplication('site');
		$params = $app->getParams('com_vmsales');

		$list = $app->getUserState($this->context . '.list');
		$ordering  = isset($list['filter_order'])     ? $list['filter_order']     : null;
		$direction = isset($list['filter_order_Dir']) ? $list['filter_order_Dir'] : null;

		$list['limit']     = (int) JFactory::getConfig()->get('list_limit', 20);
		$list['start']     = $app->input->getInt('start', 0);
		$list['ordering']  = $ordering;
		$list['direction'] = $direction;

		$app->setUserState($this->context . '.list', $list);
		$app->input->set('list', null);

		// List state information.
		parent::populateState($ordering, $direction);

        $app = JFactory::getApplication();

        $ordering  = $app->getUserStateFromRequest($this->context . '.ordercol', 'filter_order', $ordering);
        $direction = $app->getUserStateFromRequest($this->context . '.orderdirn', 'filter_order_Dir', $ordering);

        $this->setState('list.ordering', $ordering);
        $this->setState('list.direction', $direction);

        $start = $app->getUserStateFromRequest($this->context . '.limitstart', 'limitstart', 0, 'int');
        $limit = $app->getUserStateFromRequest($this->context . '.limit', 'limit', 0, 'int');

        if ($limit == 0)
        {
            $limit = $app->get('list_limit', 30);
		}

		// set custom limit of products per page
		$display_pagination = $params->get('display_pagination', '1');
		$limit = $params->get('limit_of_products_per_page', '30');
		
		if ($limit >= 0 && $display_pagination)
		{
			$this->setState('list.limit', $limit);
			$this->setState('list.start', $start);
		}
		elseif ($limit >= 0 && !$display_pagination)
		{
			$this->setState('list.limit', 0);
			$this->setState('list.start', $start);
		}
		else
		{
			$this->setState('list.limit', $limit);
			$this->setState('list.start', $start);
		}
		
		// Filter Category
		$get_category = $app->getUserStateFromRequest($this->context . '.filter.category', 'filter_category', '', 'INT');
		$this->setState('filter.category', $get_category);

		// Filter Manufacturer
		$get_manufacturer = $app->getUserStateFromRequest($this->context . '.filter.manufacturer', 'filter_manufacturer', '', 'INT');
		$this->setState('filter.manufacturer', $get_manufacturer);

		// Filter Order
		$get_orderby = $app->getUserStateFromRequest($this->context . '.filter.orderby', 'filter_orderby', '', 'STRING');
		$this->setState('filter.orderby', $get_orderby);

		// Filter Search
		$get_search = $app->getUserStateFromRequest($this->context . '.filter.search', 'filter_search', '', 'STRING');
		$this->setState('filter.search', $get_search);

		// Filter Datatype
		$get_datatype = $app->getUserStateFromRequest($this->context . '.filter.datatype', 'filter_datatype', $params->get('data_type', 'all'), 'STRING');
		$this->setState('filter.datatype', $get_datatype);	
	}

	/**
	 * Build an SQL query to load the list data.
	 *
	 * @return   JDatabaseQuery
	 *
	 * @since    1.6
	 */
	//protected function getListQuery()
	function theQuery()
	{
		// Get Params
		$app 	= JFactory::getApplication('site');
		$params = $app->getParams('com_vmsales');

		// Get Language
		$lang = new stdClass();
		$lang->current_language = JFactory::getConfig()->get('language'); // el-GR
		$this->_lang_prefix = (!empty($lang->current_language)) ? strtolower(str_replace('-', '_', $lang->current_language)) : 'en_gb';

		// query
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);
		$query
			->select(
				$this->getState('list.select', 'DISTINCT product.virtuemart_product_id')
			);
		$query->select($db->quoteName('virtuemart_product_prices.product_price'));

		// main table
		$query->from('`#__virtuemart_products` AS product');

		// Join products_lang
		$query->join('LEFT', '#__virtuemart_products_'.$this->_lang_prefix.' AS product_lang ON product.virtuemart_product_id = product_lang.virtuemart_product_id');
		
		// Join Product Prices
		$query->join('LEFT', '#__virtuemart_product_prices AS virtuemart_product_prices ON product.virtuemart_product_id = virtuemart_product_prices.virtuemart_product_id');

		// Join Product Categories
		$query->join('LEFT', '#__virtuemart_product_categories AS virtuemart_product_categories ON product.virtuemart_product_id = virtuemart_product_categories.virtuemart_product_id');

		// Join Categories
		$query->join('LEFT', '#__virtuemart_categories AS virtuemart_categories ON virtuemart_product_categories.virtuemart_category_id = virtuemart_categories.virtuemart_category_id');
		$query->where('virtuemart_categories.published = 1');

		// Join Manufacters
		$query->join('LEFT', '#__virtuemart_product_manufacturers AS virtuemart_product_manufacturers ON product.virtuemart_product_id = virtuemart_product_manufacturers.virtuemart_product_id');

		// WHERE
		$query->where('product.published = 1');
		
		// Display only parent products
		if ($params->get('type_of_products', 'only_parent') == 'only_parent')
		{
			$query->where('product.product_parent_id = 0');
		}

		// Display only featured products
		if ($params->get('display_only_featured_products', '0') == 1)
		{
			$query->where('product.product_special = 1');
		}

		// Display parent products that are out of stock
		if ($params->get('display_parent_products_that_are_out_of_stock', '1') == 0)
		{
			$query->where('IF(product.product_parent_id = 0, product.product_in_stock > 0, product.published = 1)');
		}

		// Display child products that are out of stock
		if ($params->get('display_child_products_that_are_out_of_stock', '1') == 0)
		{
			$query->where('IF(product.product_parent_id > 0, product.product_in_stock > 0, product.published = 1)');
		}

		// Filter by Category
		$category = $this->getState('filter.category');
		if (!empty($category))
		{
			$query->where('virtuemart_product_categories.virtuemart_category_id = ' . (int) $category);
		}

		// Include Categories
		$include_categories = $params->get('include_categories');
		$include_categories = (!empty($include_categories)) ? array_filter($include_categories) : '';
		if (!empty($include_categories) && is_array($include_categories))
		{
			$query->where('virtuemart_product_categories.virtuemart_category_id IN ('.implode(',', $include_categories).')');
		}

		// Exclude Categories
		$exclude_categories = $params->get('exclude_categories');
		$exclude_categories = (!empty($exclude_categories)) ? array_filter($exclude_categories) : '';
		if (!empty($exclude_categories) && is_array($exclude_categories))
		{
			$query->where('virtuemart_product_categories.virtuemart_category_id NOT IN ('.implode(',', $exclude_categories).')');
		}

		// Filter by Manufacturer
		$manufacturer = $this->getState('filter.manufacturer');
		if (!empty($manufacturer))
		{
			$query->where('virtuemart_product_manufacturers.virtuemart_manufacturer_id = ' . (int) $manufacturer);
		}
		
		// Include Manufacturers
		$include_manufacturers = $params->get('include_manufacturers');
		$include_manufacturers = (!empty($include_manufacturers)) ? array_filter($include_manufacturers) : '';
		if (!empty($include_manufacturers) && is_array($include_manufacturers))
		{
			$query->where('virtuemart_product_manufacturers.virtuemart_manufacturer_id IN ('.implode(',', $include_manufacturers).')');
		}

		// Exclude Manufacturers
		$exclude_manufacturers = $params->get('exclude_manufacturers');
		$exclude_manufacturers = (!empty($exclude_manufacturers)) ? array_filter($exclude_manufacturers) : '';
		if (!empty($exclude_manufacturers) && is_array($exclude_manufacturers))
		{
			$query->where('virtuemart_product_manufacturers.virtuemart_manufacturer_id NOT IN ('.implode(',', $exclude_manufacturers).')');
		}

		// Filter by Search
		$search = $this->getState('filter.search');
		if (!empty($search))
		{
			if (stripos($search, 'id:') === 0)
			{
				$query->where('product.virtuemart_product_id = ' . (int) substr($search, 3));
			}
			else
			{
				// get search
				$search = $db->Quote('%' . $db->escape($search, true) . '%');
				
				// build where
				$query->where('(product.virtuemart_product_id LIKE ' . $search . ' OR product.product_sku LIKE ' . $search . ' OR product_lang.product_name LIKE ' . $search . ' OR product_lang.product_s_desc LIKE ' . $search . ' OR product_lang.product_desc LIKE ' . $search . ')');
			}
		}

		$query->order("product.virtuemart_product_id DESC");

		// Unique results
		$query->group('product.virtuemart_product_id');

		return $query;
	}

	/**
	 * Method to get the total number of items.
	 *
	 * @return  int	The total number of items.
	 *
	 * @since   1.6
	 */
	public function getTotal()
	{
		if (!isset($this->_total))
		{
			$this->getItems();
		}

		return $this->_total;
	}

	/**
	 * Get an array of data items
	 *
	 * @return mixed Array of data items on success, false on failure.
	 */
	public function getItems()
	{
		
		if (!isset($this->items))
		{
			$limitstart      = $this->getState('list.start');
			$limit           = $this->getState('list.limit');
			
			$this->_db->setQuery($this->theQuery());

			try
			{
				$items = $this->_db->loadObjectList();
			}
			catch (RuntimeException $e)
			{
				$this->setError($e->getMessage());

				return false;
			}

			// Import filesystem libraries. Perhaps not necessary, but does not hurt
			jimport('joomla.filesystem.file');
		
			// BEGIN: Load VirtueMart's Api
			// load VM config
			if (!class_exists( 'VmConfig' )) 
			{
				require(JPATH_ADMINISTRATOR .'/components/com_virtuemart/helpers/config.php');
			}
			$config = VmConfig::loadConfig();
	
			// load image Class
			if (!class_exists( 'VmImage' )) 
			{
				require(JPATH_ADMINISTRATOR .'/components/com_virtuemart/helpers/image.php');
			}
	
			// Load the shopFunctionsF
			if (!class_exists ('shopFunctionsF'))
			{
				require(VMPATH_SITE . DS . 'helpers' . DS . 'shopfunctionsf.php');    
			}
	
			// Load the ShopFunctions
			if (!class_exists ('ShopFunctions'))
			{
				require(VMPATH_ADMIN . DS . 'helpers' . DS . 'shopfunctions.php');    
			}
	
			// currency
			if (!class_exists('CurrencyDisplay')) 
			{
				require(VMPATH_ADMIN . DS . 'helpers' . DS . 'currencydisplay.php');
			}
			$currency = CurrencyDisplay::getInstance();
			$this->currency_helper = $currency;
	
			// USEFUL VARS
			// echo VmConfig::get('reviews_maximum_comment_length','default');
			// JPATH_VM_SITE = JPATH_ROOT.DS.'components'.DS.'com_virtuemart' );
			// JPATH_VM_ADMINISTRATOR = JPATH_ROOT.DS.'administrator'.DS.'components'.DS.'com_virtuemart' );
	
			// Loading VM's language file
			$lang = JFactory::getLanguage();
			$extension = 'com_virtuemart';
			$base_dir = JPATH_VM_SITE;
			$lang->load($extension, $base_dir);
			
			// params
			$app 	= JFactory::getApplication('site');
			$params = $app->getParams('com_vmsales');
	
			// Get currencies
			$currencymodel = VmModel::getModel('currency', 'VirtuemartModel');
			$currencies = $currencymodel->getCurrencies();

			// Load manufacturer model
			$manModel = VmModel::getModel('manufacturer');
			$manufacturers = $manModel->getManufacturers(true, true, true);
	
			// BEGIN: Build filter manufacturers by included/excluded manufacturers
			// Include Manufacturers
			$include_manufacturers = $params->get('include_manufacturers');
			$include_manufacturers = (!empty($include_manufacturers)) ? array_filter($include_manufacturers) : array();
	
			// Exclude Manufacturers
			$exclude_manufacturers = $params->get('exclude_manufacturers');
			$exclude_manufacturers = (!empty($exclude_manufacturers)) ? array_filter($exclude_manufacturers) : array();
			
			$manufacturers_vmsales = array();
			if (!empty($include_manufacturers) || !empty($exclude_manufacturers))
			{
				foreach ($manufacturers as $manufacturer)
				{
					if (!empty($include_manufacturers))
					{
						if (in_array ($manufacturer->virtuemart_manufacturer_id, $include_manufacturers)) 
						{
							$manufacturers_vmsales[$manufacturer->virtuemart_manufacturer_id] = new stdClass();
							$manufacturers_vmsales[$manufacturer->virtuemart_manufacturer_id]->virtuemart_manufacturer_id = $manufacturer->virtuemart_manufacturer_id;
							$manufacturers_vmsales[$manufacturer->virtuemart_manufacturer_id]->mf_name = $manufacturer->mf_name;
						}

					}

					if (!empty($exclude_manufacturers))
					{
						if (!in_array ($manufacturer->virtuemart_manufacturer_id, $exclude_manufacturers)) 
						{
							$manufacturers_vmsales[$manufacturer->virtuemart_manufacturer_id] = new stdClass();
							$manufacturers_vmsales[$manufacturer->virtuemart_manufacturer_id]->virtuemart_manufacturer_id = $manufacturer->virtuemart_manufacturer_id;
							$manufacturers_vmsales[$manufacturer->virtuemart_manufacturer_id]->mf_name = $manufacturer->mf_name;
						}
					}
					
				}
			}
			$manufacturers = !empty($manufacturers_vmsales) ? $manufacturers_vmsales : $manufacturers;
			// END: Build filter manufacturers by included/excluded manufacturers
			// Filter manufacturer
			$manufacturers_data = array();
			$manufacturers_data[] = JHTML::_('select.option', '', JText::_('VMSALES_ALL_MANUFACTURERS'), 'virtuemart_manufacturer_id', 'mf_name' );
			$manufacturers_data = array_merge( $manufacturers_data, $manufacturers);
			$this->manufacturers_list = JHTML::_('select.genericlist',
								$manufacturers_data, 
								'filter[manufacturer]', 
								'class="uk-select uk-width-7-10 uk-width-1-1@l uk-width-1-1@m uk-width-1-1@s"',
								'virtuemart_manufacturer_id', 
								'mf_name', 
								$this->getState('filter.manufacturer'), 
								'filter_manufacturer', 
								true);

			// Filter category
			$selected_category = $this->getState('filter.category');
			$this->category_list_tree = $this->categoryListTreeVMSales($selected_category);

			// quantity
			$quantityArray = vRequest::getInt ('quantity', array()); //is sanitized then
			$quantity = 1;
			if (!empty($quantityArray[0])) {
				$quantity = $quantityArray[0];
			}
	
			// get data type
			$this->datatype = $this->getState('filter.datatype'); // with_discount, without_discount, all
	
			// products
			$this->products_arr = array();
			$this->products_with_discount_arr = array();
			$this->products_without_discount_arr = array();
			$this->all_products_arr = array();
			if (!empty($items)):

				$product_model = VmModel::getModel('product');

				$i=0;
				foreach ($items as $item):
	
					// product
					$product = $product_model->getProduct($item->virtuemart_product_id,TRUE,TRUE,TRUE,$quantity);

					// BEGIN: percentage discount (e.g. 20%)
					$this->products_arr[$i]['discount_percentage_text'] = 0;

					if (isset($product->prices['DATax']) && !empty($product->prices['DATax']))
					{
						$discount_amount = $product->prices['DATax'];
						$discount_amount = array_values($discount_amount);
						
						if (isset($discount_amount[0][1]))
						{
							$discount_percentage_text = (int) number_format($discount_amount[0][1], 0);
						}
						else
						{
							$discount_percentage_text = 0;
						}

						$this->products_arr[$i]['discount_percentage_text'] = (!empty($discount_percentage_text)) ?  $discount_percentage_text : 0;
					}
					elseif (isset($product->prices['DBTax']) && !empty($product->prices['DBTax']))
					{
						$discount_amount = $product->prices['DBTax'];
						$discount_amount = array_values($discount_amount);

						if (isset($discount_amount[0][1]))
						{
							$discount_percentage_text = (int) number_format($discount_amount[0][1], 0);
						}
						else
						{
							$discount_percentage_text = 0;
						}

						$this->products_arr[$i]['discount_percentage_text'] = (!empty($discount_percentage_text)) ?  $discount_percentage_text : 0;				
					}
					// END: percentage discount (e.g. 20%)
					
					if (!empty($product))
					{
						// products with discount in array
						$discount_amount = (isset($product->allPrices[0]['discountAmount'])) ? (float) abs($product->allPrices[0]['discountAmount']) : 0;

						if ($discount_amount > 0)
						{
							$this->products_with_discount_arr[] = $product->virtuemart_product_id;
							$this->all_products_arr[] = $product->virtuemart_product_id;
						}
						else
						{
							$this->products_without_discount_arr[] = $product->virtuemart_product_id;
							$this->all_products_arr[] = $product->virtuemart_product_id;
						}
	
						// first
						$this->products_arr[$i]['salesPrice_v'] = (isset($product->allPrices[0]['salesPrice'])) ? (float) $product->allPrices[0]['salesPrice'] : 0;
						$this->products_arr[$i]['discountAmount_v'] = (isset($product->allPrices[0]['discountAmount'])) ? (float) abs($product->allPrices[0]['discountAmount']) : 0;
						$this->products_arr[$i]['discountAmount_digit'] = (isset($product->allPrices[0]['discountAmount'])) ? (float) $product->allPrices[0]['discountAmount'] : 0;
						
						// product details
						$this->products_arr[$i]['virtuemart_product_id'] = $product->virtuemart_product_id;
						$this->products_arr[$i]['product_sku'] = $product->product_sku;
						$this->products_arr[$i]['product_name'] = $product->product_name;
						$this->products_arr[$i]['product_s_desc'] = $product->product_s_desc;
						$this->products_arr[$i]['product_desc'] = $product->product_desc;
						$this->products_arr[$i]['product_link'] = $product->link;
						$this->products_arr[$i]['product_parent_id'] = $product->product_parent_id;
						$this->products_arr[$i]['modified_on'] = $product->modified_on;
						$this->products_arr[$i]['product_price'] = $item->product_price;
						$this->products_arr[$i]['product_weight'] = $product->product_weight;

						// prices
						$allPrices = $product->allPrices;
						foreach ($allPrices as $price)
						{
							$this->products_arr[$i]['basePriceVariant__DIGIT_ONLY'] = (isset($price['basePriceVariant']) ? $price['basePriceVariant'] : '');
						}
	
						// prices
						$this->products_arr[$i]['basePriceVariant'] = $currency->createPriceDiv('basePriceVariant', '', $product->prices, true);
						$this->products_arr[$i]['variantModification'] = $currency->createPriceDiv('variantModification', '', $product->prices, true);
						$this->products_arr[$i]['basePriceWithTax'] = $currency->createPriceDiv('basePriceWithTax', '', $product->prices, true);
						$this->products_arr[$i]['salesPriceWithDiscount'] = $currency->createPriceDiv('salesPriceWithDiscount', '', $product->prices, true);
						$this->products_arr[$i]['salesPrice'] = $currency->createPriceDiv('salesPrice', '', $product->prices, true);
						$this->products_arr[$i]['salesPriceTt'] = $currency->createPriceDiv('salesPriceTt', '', $product->prices, true);
						$this->products_arr[$i]['discountedPriceWithoutTax'] = $currency->createPriceDiv('discountedPriceWithoutTax', '', $product->prices, true);
						$this->products_arr[$i]['discountedPriceWithoutTaxTt'] = $currency->createPriceDiv('discountedPriceWithoutTaxTt', '', $product->prices, true);
						$this->products_arr[$i]['priceWithoutTax'] = $currency->createPriceDiv('priceWithoutTax', '', $product->prices, true);
						$this->products_arr[$i]['priceWithoutTaxTt'] = $currency->createPriceDiv('priceWithoutTaxTt', '', $product->prices, true);
						$this->products_arr[$i]['discountAmount'] = $currency->createPriceDiv('discountAmount', '', $product->prices, true);
						$this->products_arr[$i]['discountAmountTt'] = $currency->createPriceDiv('discountAmountTt', '', $product->prices, true);
						$this->products_arr[$i]['taxAmount'] = $currency->createPriceDiv('taxAmount', '', $product->prices, true);
						$this->products_arr[$i]['taxAmountTt'] = $currency->createPriceDiv('taxAmountTt', '', $product->prices, true);
						$this->products_arr[$i]['unitPrice'] = $currency->createPriceDiv('unitPrice', '', $product->prices, true);
	
						// images
						$product_model->addImages($product);
						if (!empty($product->images)) 
						{
							$this->products_arr[$i]['productImages'] = $product->images;
							foreach($product->images as $img)
							{
								if (!empty($img->file_url))
								{
									$this->products_arr[$i]['images'][] = $img->file_url;
								}
							}
						}

						// manufacturers
						if (!empty($product->virtuemart_manufacturer_id)) {
							foreach($product->virtuemart_manufacturer_id as $manufacturer_id) 
							{
								$manufacturer = $manModel->getManufacturer( $manufacturer_id );
								$manufacturer_link = JRoute::_('index.php?option=com_virtuemart&view=manufacturer&virtuemart_manufacturer_id='.$manufacturer->virtuemart_manufacturer_id.'&tmpl=component', FALSE);
								$manufacturer_id = $manufacturer->virtuemart_manufacturer_id;
								$manufacturer_name = $manufacturer->mf_name;
								$this->products_arr[$i]['manufacturers'][] = array('name' => $manufacturer_name, 'link' => $manufacturer_link, 'id' => $manufacturer_id);
							}
						}
	
						// categories
						if (!empty($product->virtuemart_category_id)) 
						{
							$categoryItems = $product->categoryItem;
	
							foreach ($categoryItems as $categoryItem)
							{
								$category_name = $categoryItem['category_name'];
								$category_url = JRoute::_('index.php?option=com_virtuemart&view=category&virtuemart_category_id=' . $categoryItem['virtuemart_category_id'], FALSE);
								$this->products_arr[$i]['categories'][] = array('name' => $category_name, 'link' => $category_url, 'virtuemart_category_id' => $categoryItem['virtuemart_category_id']);
							}
	
						}
						
					$i++;
					}
				endforeach;

				$items = $this->products_arr;

    	        // unset products from array if the datatype is with_discount or without_discount
				$i = 0;
				if (!empty($items))
				{
					foreach ($items as $item)
					{
						switch ($this->datatype) 
						{
							case 'with_discount':
								if (!in_array($items[$i]['virtuemart_product_id'], $this->getProductsWithDiscount()))
								{
									unset($items[$i]);
								}
								break;
							case 'without_discount':
								if (!in_array($items[$i]['virtuemart_product_id'], $this->getProductsWithoutDiscount()))
								{
									unset($items[$i]);
								}
								break;
							default:
								// all products
								break;
						}
					$i++;
					}
				}
			
				$this->_total = count($items);
			endif;
			// END: Load VirtueMart's Api

			$this->items = array_slice($items, $limitstart, $limit ?: null);
		}
		
		return $this->items;
	}

	public function getProductsWithDiscount()
	{
		return $this->products_with_discount_arr;
	}

	public function getProductsWithoutDiscount()
	{
		return $this->products_without_discount_arr;
	}

	public function getManufacturers()
	{
		return $this->manufacturers_list;
	}

	public function getCategories()
	{
		return $this->category_list_tree;
	}

	public function getCurrencyHelper()
	{
		return $this->currency_helper;
	}

	public static $counter = 0;
	public static $categoryTree = array();

	static public function categoryListTreeVMSales ($selectedCategories = array(), $cid = 0, $level = 0, $disabledFields = array()) {

		if(!is_array($selectedCategories)){
			$selectedCategories = array($selectedCategories);
		}
		$hash = crc32(implode('.',$selectedCategories).':'.$cid.':'.$level.implode('.',$disabledFields));
		if (empty(self::$categoryTree[$hash])) {

			$cache = VmConfig::getCache ('com_virtuemart_cats');
			$cache->setCaching (1);

			$vendorId = vmAccess::isSuperVendor();
			self::$categoryTree[$hash] = $cache->call (array('vmsalesModelvmsales', 'categoryListTreeLoopVMSales'), $selectedCategories, $cid, $level, $disabledFields,VmConfig::isSite(),$vendorId,VmConfig::$vmlang);
		}

		return self::$categoryTree[$hash];
	}
	
	/**
	 * Creates structured option fields for all categories
	 *
	 * @todo: Connect to vendor data
	 * @author Max Milbers, jseros
	 * @param array 	$selectedCategories All category IDs that will be pre-selected
	 * @param int 		$cid 		Internally used for recursion
	 * @param int 		$level 		Internally used for recursion
	 * @return string 	$category_tree HTML: Category tree list
	 */
	static public function categoryListTreeLoopVMSales ($selectedCategories = array(), $cid = 0, $level = 0, $disabledFields = array(), $isSite, $vendorId, $vmlang,$categoryParentName='') 
	{
		// BEGIN: Build filter category by included/excluded categories
		$app 	= JFactory::getApplication('site');
		$params = $app->getParams('com_vmsales');
		$include_categories = $params->get('include_categories');
		$include_categories = (!empty($include_categories)) ? array_filter($include_categories) : array();
		$exclude_categories = $params->get('exclude_categories');
		$exclude_categories = (!empty($exclude_categories)) ? array_filter($exclude_categories) : array();
		// END: Build filter category by included/excluded categories

		vmSetStartTime('categoryListTreeLoopVMSales');
		static $categoryTree = '';
		if($level==0) {
			$categoryTree = '';
			self::$counter = 0;
		}
		self::$counter++;
		$categoryModel = VmModel::getModel ('category');
		$level++;

		$categoryModel->_noLimit = TRUE;

		$records = $categoryModel->getChildCategoryList($vendorId, $cid );

		$selected = "";
		if (!empty($records)) {
			foreach ($records as $key => $category) {

				if ($category->virtuemart_category_id != $cid) {
					if (in_array ($category->virtuemart_category_id, $selectedCategories)) {
						$selected = 'selected=\"selected\"';
					} else {
						$selected = '';
					}

					$disabled = '';
					if (in_array ($category->virtuemart_category_id, $disabledFields)) {
						$disabled = 'disabled="disabled"';
					}

					// BEGIN: Web357 - Build filter category by included/excluded categories
					$style = '';
					if (!empty($include_categories))
					{
						if (in_array($category->virtuemart_category_id, $include_categories)) 
						{
							$style = 'style="display:block"';
						}
						else
						{
							$style = 'style="display:none"';
						}
					}
					if (!empty($exclude_categories))
					{
						if (in_array($category->virtuemart_category_id, $exclude_categories)) 
						{
							$style = 'style="display:none"';
						}
					}
					// END: Web357 - Build filter category by included/excluded categories

					if ($disabled != '' && stristr ($_SERVER['HTTP_USER_AGENT'], 'msie')) {
						//IE7 suffers from a bug, which makes disabled option fields selectable
					} else {
						$categoryTree .= '<option ' . $selected . ' ' . $style .' ' . $disabled . ' value="' . $category->virtuemart_category_id . '">';
						$categoryName = $category->category_name;
						if(VmConfig::get('full_catname_tree',0)) {
							if (!empty($categoryParentName)) {
								$categoryName =$categoryParentName.' | '.$category->category_name;
							}
						} else {
							$categoryTree .= str_repeat (' - ', ($level - 1));
						}
						$categoryTree .= $categoryName . '</option>';
					}
				}
			
				if ($category->has_children) {
					self::categoryListTreeLoopVMSales ($selectedCategories, $category->virtuemart_category_id, $level, $disabledFields,$isSite, $vendorId, $vmlang, $categoryName);
				}
			}
		}

		vmTime('categoryListTreeLoopVMSales','categoryListTreeLoopVMSales');
		return $categoryTree;
	}

}